@@ -1,6 +1,6 @@
 // Smuxi - Smart MUltipleXed Irc
 // 
-// Copyright (c) 2009-2011 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2009-2013 Mirco Bauer <meebey@meebey.net>
 // 
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
 // 
@@ -22,6 +22,7 @@ using System;
 using System.Net;
 using System.Net.Security;
 using System.Web;
+using System.Linq;
 using System.Security.Cryptography.X509Certificates;
 using System.Threading;
 using System.Collections.Generic;
@@ -81,6 +82,9 @@ namespace Smuxi.Engine
         int                     ErrorResponseCount { get; set; }
         const int               MaxErrorResponseCount = 3;
 
+        TwitterStatus[]         StatusIndex { get; set; }
+        int                     StatusIndexOffset { get; set; }
+
         public override string NetworkID {
             get {
                 if (f_TwitterUser == null) {
@@ -147,15 +151,14 @@ namespace Smuxi.Engine
             );
             f_DirectMessagesChat.ApplyConfig(Session.UserConfig);
             f_GroupChats.Add(f_DirectMessagesChat);
+
+            StatusIndex = new TwitterStatus[99];
         }
 
         public override void Connect(FrontendManager fm, ServerModel server)
         {
             Trace.Call(fm, server);
 
-            if (fm == null) {
-                throw new ArgumentNullException("fm");
-            }
             if (server == null) {
                 throw new ArgumentNullException("server");
             }
@@ -206,10 +209,13 @@ namespace Smuxi.Engine
                 }
             }
 
-            string msg;
-            msg = String.Format(_("Connecting to Twitter..."));
-            fm.SetStatus(msg);
-            Session.AddTextToChat(f_ProtocolChat, "-!- " + msg);
+            string msgStr = _("Connecting to Twitter...");
+            if (fm != null) {
+                fm.SetStatus(msgStr);
+            }
+            var msg = CreateMessageBuilder().
+                AppendEventPrefix().AppendText(msgStr).ToMessage();
+            Session.AddMessageToChat(Chat, msg);
             try {
                 var key = GetApiKey();
                 f_OAuthTokens = new OAuthTokens();
@@ -282,10 +288,16 @@ namespace Smuxi.Engine
 #if LOG4NET
                 f_Logger.Error("Connect(): Exception", ex);
 #endif
-                fm.SetStatus(_("Connection failed!"));
-                Session.AddTextToChat(f_ProtocolChat,
-                    "-!- " + _("Connection failed! Reason: ") + ex.Message
-                );
+                if (fm != null) {
+                    fm.SetStatus(_("Connection failed!"));
+                }
+                msg = CreateMessageBuilder().
+                    AppendEventPrefix().
+                    AppendErrorText(
+                        _("Connection failed! Reason: {0}"),
+                        ex.Message).
+                    ToMessage();
+                Session.AddMessageToChat(Chat, msg);
                 return;
             }
 
@@ -298,34 +310,55 @@ namespace Smuxi.Engine
                     }
                     
                     var message = _("Fetching user details from Twitter, please wait...");
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message);
+                    msg = CreateMessageBuilder().
+                        AppendEventPrefix().AppendText(message).ToMessage();
+                    Session.AddMessageToChat(Chat, msg);
 
                     UpdateUser();
 
                     message = _("Finished fetching user details.");
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message);
+                    msg = CreateMessageBuilder().
+                        AppendEventPrefix().AppendText(message).ToMessage();
+                    Session.AddMessageToChat(Chat, msg);
 
                     f_IsConnected = true;
-                    fm.UpdateNetworkStatus();
-                    msg =_("Successfully connected to Twitter.");
-                    fm.SetStatus(msg);
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + msg);
+                    message =_("Successfully connected to Twitter.");
+                    if (fm != null) {
+                        fm.UpdateNetworkStatus();
+                        fm.SetStatus(message);
+                    }
+
+                    msg = CreateMessageBuilder().
+                        AppendEventPrefix().AppendText(message).ToMessage();
+                    Session.AddMessageToChat(Chat, msg);
                     f_Listening = true;
 
                     f_FriendsTimelineChat.PersonCount = 
                     f_RepliesChat.PersonCount = 
                     f_DirectMessagesChat.PersonCount = (int) f_TwitterUser.NumberOfFriends;
+
+                    OnConnected(EventArgs.Empty);
+
                 } catch (Exception ex) {
                     var message = _("Failed to fetch user details from Twitter. Reason: ");
 #if LOG4NET
                     f_Logger.Error("Connect(): " + message, ex);
 #endif
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message + ex.Message);
+                    msg = CreateMessageBuilder().
+                        AppendEventPrefix().
+                        AppendErrorText(message + ex.Message).
+                        ToMessage();
+                    Session.AddMessageToChat(Chat, msg);
 
-                    fm.SetStatus(_("Connection failed!"));
-                    Session.AddTextToChat(f_ProtocolChat,
-                        "-!- " + _("Connection failed! Reason: ") + ex.Message
-                    );
+                    if (fm != null) {
+                        fm.SetStatus(_("Connection failed!"));
+                    }
+                    msg = CreateMessageBuilder().
+                        AppendEventPrefix().
+                        AppendErrorText(_("Connection failed! Reason: {0}"),
+                                        ex.Message).
+                        ToMessage();
+                    Session.AddMessageToChat(Chat, msg);
                 }
             });
             ThreadPool.QueueUserWorkItem(delegate {
@@ -337,19 +370,31 @@ namespace Smuxi.Engine
                         Thread.Sleep(1000);
                     }
 
-                    var message = _("Fetching friends from Twitter, please wait...");
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message);
+                    msg = CreateMessageBuilder().
+                        AppendEventPrefix().
+                        AppendText(
+                            _("Fetching friends from Twitter, please wait...")
+                        ).
+                        ToMessage();
+                    Session.AddMessageToChat(Chat, msg);
 
                     UpdateFriends();
 
-                    message = _("Finished fetching friends.");
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message);
+                    msg = CreateMessageBuilder().
+                        AppendEventPrefix().
+                        AppendText(_("Finished fetching friends.")).
+                        ToMessage();
+                    Session.AddMessageToChat(Chat, msg);
                 } catch (Exception ex) {
                     var message = _("Failed to fetch friends from Twitter. Reason: ");
 #if LOG4NET
                     f_Logger.Error("Connect(): " + message, ex);
 #endif
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message + ex.Message);
+                    msg = CreateMessageBuilder().
+                        AppendEventPrefix().
+                        AppendErrorText(message + ex.Message).
+                        ToMessage();
+                    Session.AddMessageToChat(Chat, msg);
                 }
             });
 
@@ -530,10 +575,17 @@ namespace Smuxi.Engine
         {
             Trace.Call(fm, chat);
 
-            if (chat.ChatType == ChatType.Group) {
-               TwitterChatType twitterChatType = (TwitterChatType)
-                    Enum.Parse(typeof(TwitterChatType), chat.ID);
-               switch (twitterChatType) {
+            TwitterChatType? chatType = null;
+            try {
+                chatType = (TwitterChatType) Enum.Parse(
+                    typeof(TwitterChatType),
+                    chat.ID
+                );
+            } catch (ArgumentException) {
+            }
+            if (chat.ChatType == ChatType.Group &&
+                chatType.HasValue) {
+               switch (chatType.Value) {
                     case TwitterChatType.FriendsTimeline:
                         if (f_UpdateFriendsTimelineThread != null &&
                             f_UpdateFriendsTimelineThread.IsAlive) {
@@ -579,6 +631,32 @@ namespace Smuxi.Engine
                             CommandMessage(command);
                             handled = true;
                             break;
+                        case "timeline":
+                            CommandTimeline(command);
+                            handled = true;
+                            break;
+                        case "follow":
+                            CommandFollow(command);
+                            handled = true;
+                            break;
+                        case "unfollow":
+                            CommandUnfollow(command);
+                            handled = true;
+                            break;
+                        case "search":
+                        case "join":
+                            CommandSearch(command);
+                            handled = true;
+                            break;
+                        case "rt":
+                        case "retweet":
+                            CommandRetweet(command);
+                            handled = true;
+                            break;
+                        case "reply":
+                            CommandReply(command);
+                            handled = true;
+                            break;
                     }
                 }
                 switch (command.Command) {
@@ -623,18 +701,23 @@ namespace Smuxi.Engine
             // TRANSLATOR: this line is used as a label / category for a
             // list of commands below
             builder.AppendHeader(_("Twitter Commands"));
-            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
+            Session.AddMessageToFrontend(cd, builder.ToMessage());
 
             string[] help = {
                 "connect twitter username",
                 "pin pin-number",
+                "follow screen-name|user-id",
+                "unfollow screen-name|user-id",
+                "search keyword",
+                "retweet/rt index-number|tweet-id",
+                "reply index-number|tweet-id message",
             };
 
             foreach (string line in help) {
                 builder = CreateMessageBuilder();
                 builder.AppendEventPrefix();
                 builder.AppendText(line);
-                cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
+                Session.AddMessageToFrontend(cd, builder.ToMessage());
             }
         }
 
@@ -766,30 +849,35 @@ namespace Smuxi.Engine
 
         public void CommandSay(CommandModel cmd)
         {
-            FrontendManager fm = cmd.FrontendManager;
             if (cmd.Chat.ChatType == ChatType.Group) {
                 TwitterChatType twitterChatType = (TwitterChatType)
                     Enum.Parse(typeof(TwitterChatType), cmd.Chat.ID);
                 switch (twitterChatType) {
                     case TwitterChatType.FriendsTimeline:
-                    case TwitterChatType.Replies:
+                    case TwitterChatType.Replies: {
                         try {
                             PostUpdate(cmd.Data);
                         } catch (Exception ex) {
-                            fm.AddTextToChat(cmd.Chat, "-!- " +
-                                String.Format(_("Could not update status - Reason: {0}"),
-                                              ex.Message)
-                            );
+                            var msg = CreateMessageBuilder().
+                                AppendEventPrefix().
+                                AppendErrorText(
+                                    _("Could not update status - Reason: {0}"),
+                                    ex.Message).
+                                ToMessage();
+                            Session.AddMessageToFrontend(cmd, msg);
                         }
                         break;
-                    case TwitterChatType.DirectMessages:
-                        fm.AddTextToChat(
-                            cmd.Chat,
-                            "-!- " +
-                            _("Cannot send message - no target specified. "+
-                              "Use: /msg $nick message")
-                        );
+                    }
+                    case TwitterChatType.DirectMessages: {
+                        var msg = CreateMessageBuilder().
+                            AppendEventPrefix().
+                            AppendErrorText(
+                                _("Cannot send message - no target specified. " +
+                                  "Use: /msg $nick message")).
+                            ToMessage();
+                        Session.AddMessageToFrontend(cmd, msg);
                         break;
+                    }
                 }
             } else if (cmd.Chat.ChatType == ChatType.Person) {
                 try {
@@ -798,19 +886,74 @@ namespace Smuxi.Engine
 #if LOG4NET
                     f_Logger.Error(ex);
 #endif
-                    fm.AddTextToChat(cmd.Chat, "-!- " +
-                        String.Format(_("Could not send message - Reason: {0}"),
-                                      ex.Message)
-                    );
+                    var msg = CreateMessageBuilder().
+                        AppendEventPrefix().
+                        AppendErrorText(
+                            _("Could not send message - Reason: {0}"),
+                            ex.Message).
+                        ToMessage();
+                    Session.AddMessageToFrontend(cmd, msg);
                 }
             } else {
                 // ignore protocol chat
             }
         }
 
+        public void CommandTimeline(CommandModel cmd)
+        {
+            if (cmd.DataArray.Length < 2) {
+                NotEnoughParameters(cmd);
+                return;
+            }
+
+            string keyword = cmd.Parameter;
+            string[] users = cmd.Parameter.Split(',');
+
+            string chatName = users.Length > 1 ? _("Other timelines") : "@" + users[0];
+            ChatModel chat;
+
+            if (users.Length > 1) {
+                chat = Session.CreateChat<GroupChatModel>(keyword, chatName, this);
+            } else {
+                var userResponse = TwitterUser.Show(f_OAuthTokens, users [0], f_OptionalProperties);
+                CheckResponse(userResponse);
+                var person = GetPerson(userResponse.ResponseObject);
+                chat = Session.CreatePersonChat(person, person.ID + "/timeline",
+                                                chatName, this);
+            }
+
+            var statuses = new List<TwitterStatus>();
+            foreach (var user in users) {
+                var opts = CreateOptions<UserTimelineOptions>();
+                opts.ScreenName = user;
+                var statusCollectionResponse = TwitterTimeline.UserTimeline(f_OAuthTokens, opts);
+                CheckResponse(statusCollectionResponse);
+
+                foreach (var status in statusCollectionResponse.ResponseObject) {
+                    statuses.Add(status);
+                }
+            }
+
+            var sortedStatuses = SortTimeline(statuses);
+            foreach (var status in sortedStatuses) {
+                AddIndexToStatus(status);
+                var msg = CreateMessageBuilder().
+                    Append(status, GetPerson(status.User)).ToMessage();
+                chat.MessageBuffer.Add(msg);
+                var userId = status.User.Id.ToString();
+                var groupChat = chat as GroupChatModel;
+                if (groupChat != null) {
+                    if (!groupChat.UnsafePersons.ContainsKey(userId)) {
+                        groupChat.UnsafePersons.Add(userId, GetPerson(status.User));
+                    }
+                }
+            }
+            Session.AddChat(chat);
+            Session.SyncChat(chat);
+        }
+
         public void CommandMessage(CommandModel cmd)
         {
-            FrontendManager fm = cmd.FrontendManager;
             string nickname;
             if (cmd.DataArray.Length >= 2) {
                 nickname = cmd.DataArray[1];
@@ -822,9 +965,12 @@ namespace Smuxi.Engine
             var response = TwitterUser.Show(f_OAuthTokens, nickname,
                                             f_OptionalProperties);
             if (response.Result != RequestResult.Success) {
-                fm.AddTextToChat(cmd.Chat, "-!- " +
-                    _("Could not send message - the specified user does not exist.")
-                );
+                var msg = CreateMessageBuilder().
+                    AppendEventPrefix().
+                    AppendErrorText(_("Could not send message - the " +
+                                      "specified user does not exist.")).
+                    ToMessage();
+                Session.AddMessageToFrontend(cmd, msg);
                 return;
             }
             var user = response.ResponseObject;
@@ -835,29 +981,193 @@ namespace Smuxi.Engine
                 try {
                     SendMessage(user.ScreenName, message);
                 } catch (Exception ex) {
-                    fm.AddTextToChat(chat, "-!- " +
-                        String.Format(_("Could not send message - Reason: {0}"),
-                                      ex.Message)
-                    );
+                    var msg = CreateMessageBuilder().
+                        AppendEventPrefix().
+                        AppendErrorText(
+                            _("Could not send message - Reason: {0}"),
+                            ex.Message).
+                        ToMessage();
+                    Session.AddMessageToFrontend(cmd.FrontendManager, chat, msg);
                 }
             }
-         }
+        }
 
-        private List<TwitterStatus> SortTimeline(TwitterStatusCollection timeline)
+        public void CommandFollow(CommandModel cmd)
+        {
+            if (cmd.DataArray.Length < 2) {
+                NotEnoughParameters(cmd);
+                return;
+            }
+
+            var chat = cmd.Chat as GroupChatModel;
+            if (chat == null) {
+                return;
+            }
+
+            var options = CreateOptions<CreateFriendshipOptions>();
+            options.Follow = true;
+            decimal userId;
+            TwitterResponse<TwitterUser> res;
+            if (Decimal.TryParse(cmd.Parameter, out userId)) {
+                // parameter is an ID
+                res = TwitterFriendship.Create(f_OAuthTokens, userId, options);
+            } else {
+                // parameter is a screen name
+                var screenName = cmd.Parameter;
+                res = TwitterFriendship.Create(f_OAuthTokens, screenName, options);
+            }
+            CheckResponse(res);
+            var person = CreatePerson(res.ResponseObject);
+            if (chat.GetPerson(person.ID) == null) {
+                Session.AddPersonToGroupChat(chat, person);
+            }
+        }
+
+        public void CommandUnfollow(CommandModel cmd)
+        {
+            if (cmd.DataArray.Length < 2) {
+                NotEnoughParameters(cmd);
+                return;
+            }
+
+            var chat = cmd.Chat as GroupChatModel;
+            if (chat == null) {
+                return;
+            }
+
+            PersonModel person;
+            var persons = chat.Persons;
+            if (persons.TryGetValue(cmd.Parameter, out person)) {
+                // parameter is an ID
+                decimal userId;
+                Decimal.TryParse(cmd.Parameter, out userId);
+                var res = TwitterFriendship.Delete(f_OAuthTokens, userId, f_OptionalProperties);
+                CheckResponse(res);
+            } else {
+                // parameter is a screen name
+                var screenName = cmd.Parameter;
+                person = persons.Single((arg) => arg.Value.IdentityName == screenName).Value;
+                var res = TwitterFriendship.Delete(f_OAuthTokens, screenName, f_OptionalProperties);
+                CheckResponse(res);
+            }
+            Session.RemovePersonFromGroupChat(chat, person);
+        }
+
+        public bool IsHomeTimeLine(ChatModel chatModel)
+        {
+            return chatModel.Equals(f_FriendsTimelineChat);
+        }
+
+        private List<TwitterStatus> SortTimeline(IList<TwitterStatus> timeline)
         {
             List<TwitterStatus> sortedTimeline =
                 new List<TwitterStatus>(
-                    timeline.Count
+                    timeline
                 );
-            foreach (TwitterStatus status in timeline) {
-                sortedTimeline.Add(status);
-            }
             sortedTimeline.Sort(
                 (a, b) => (a.CreatedDate.CompareTo(b.CreatedDate))
             );
             return sortedTimeline;
         }
 
+        public void CommandSearch(CommandModel cmd)
+        {
+            if (cmd.DataArray.Length < 2) {
+                NotEnoughParameters(cmd);
+                return;
+            }
+
+            var keyword = cmd.Parameter;
+            var chatName = String.Format(_("Search {0}"), keyword);
+            var chat = Session.CreateChat<GroupChatModel>(keyword, chatName, this);
+            Session.AddChat(chat);
+            var options = CreateOptions<SearchOptions>();
+            options.Count = 50;
+            var response = TwitterSearch.Search(f_OAuthTokens, keyword, options);
+            CheckResponse(response);
+            var search = response.ResponseObject;
+            var sortedSearch = SortTimeline(search);
+            foreach (var status in sortedSearch) {
+                AddIndexToStatus(status);
+                var msg = CreateMessageBuilder().
+                    Append(status, GetPerson(status.User)).
+                    ToMessage();
+                chat.MessageBuffer.Add(msg);
+                var userId = status.User.Id.ToString();
+                if (!chat.UnsafePersons.ContainsKey(userId)) {
+                    chat.UnsafePersons.Add(userId, GetPerson(status.User));
+                }
+            }
+            Session.SyncChat(chat);
+        }
+
+        public void CommandRetweet(CommandModel cmd)
+        {
+            if (cmd.DataArray.Length < 2) {
+                NotEnoughParameters(cmd);
+                return;
+            }
+
+
+            TwitterStatus status = null;
+            int indexId;
+            if (Int32.TryParse(cmd.Parameter, out indexId)) {
+                status = GetStatusFromIndex(indexId);
+            }
+
+            decimal statusId;
+            if (status == null) {
+                if (!Decimal.TryParse(cmd.Parameter, out statusId)) {
+                    return;
+                }
+            } else {
+                statusId = status.Id;
+            }
+            var response = TwitterStatus.Retweet(f_OAuthTokens, statusId, f_OptionalProperties);
+            CheckResponse(response);
+            status = response.ResponseObject;
+
+            var msg = CreateMessageBuilder().
+                Append(status, GetPerson(status.User)).
+                ToMessage();
+            Session.AddMessageToChat(f_FriendsTimelineChat, msg);
+        }
+
+        public void CommandReply(CommandModel cmd)
+        {
+            if (cmd.DataArray.Length < 3) {
+                NotEnoughParameters(cmd);
+                return;
+            }
+
+            var id = cmd.DataArray[1];
+            TwitterStatus status = null;
+            int indexId;
+            if (Int32.TryParse(id, out indexId)) {
+                status = GetStatusFromIndex(indexId);
+            }
+
+            decimal statusId;
+            if (status == null) {
+                if (!Decimal.TryParse(id, out statusId)) {
+                    return;
+                }
+                var response = TwitterStatus.Show(f_OAuthTokens, statusId,
+                                                  f_OptionalProperties);
+                CheckResponse(response);
+                status = response.ResponseObject;
+            }
+
+            var text = String.Join(" ", cmd.DataArray.Skip(2).ToArray());
+            // the screen name must be somewhere in the message for replies
+            if (!text.Contains("@" + status.User.ScreenName)) {
+                text = String.Format("@{0} {1}", status.User.ScreenName, text);
+            }
+            var options = CreateOptions<StatusUpdateOptions>();
+            options.InReplyToStatusId = status.Id;
+            PostUpdate(text, options);
+        }
+
         private List<TwitterDirectMessage> SortTimeline(TwitterDirectMessageCollection timeline)
         {
             var sortedTimeline = new List<TwitterDirectMessage>(timeline.Count);
@@ -912,8 +1222,14 @@ namespace Smuxi.Engine
 #if LOG4NET
                 f_Logger.Error("UpdateFriendsTimelineThread(): Exception", ex);
 #endif
-                string msg =_("An error occurred while fetching the friends timeline from Twitter. Reason: ");
-                Session.AddTextToChat(f_ProtocolChat, "-!- " + msg + ex.Message);
+                var msg = CreateMessageBuilder().
+                    AppendEventPrefix().
+                    AppendErrorText(
+                        _("An error occurred while fetching the friends " +
+                          "timeline from Twitter. Reason: {0}"),
+                        ex.Message).
+                    ToMessage();
+                Session.AddMessageToChat(Chat, msg);
             } finally {
 #if LOG4NET
                 f_Logger.Debug("UpdateFriendsTimelineThread(): finishing thread.");
@@ -955,25 +1271,24 @@ namespace Smuxi.Engine
 
             List<TwitterStatus> sortedTimeline = SortTimeline(timeline);
             foreach (TwitterStatus status in sortedTimeline) {
-                String text;
-                // LAME: Twitter lies in the truncated field and says it's not
-                // truncated while it is, thus always use retweet_status if
-                // available
-                if (status.RetweetedStatus != null) {
-                    text = String.Format(
-                        "RT @{0}: {1}",
-                        status.RetweetedStatus.User.ScreenName,
-                        status.RetweetedStatus.Text
+                AddIndexToStatus(status);
+                var msg = CreateMessageBuilder().
+                    Append(status, GetPerson(status.User)).
+                    ToMessage();
+                Session.AddMessageToChat(f_FriendsTimelineChat, msg);
+
+                if (status.User.Id.ToString() == Me.ID) {
+                    OnMessageSent(
+                        new MessageEventArgs(f_FriendsTimelineChat, msg, null,
+                                             status.InReplyToScreenName ?? String.Empty)
                     );
                 } else {
-                    text = status.Text;
+                    OnMessageReceived(
+                        new MessageEventArgs(f_FriendsTimelineChat, msg,
+                                             status.User.ScreenName,
+                                             status.InReplyToScreenName ?? String.Empty)
+                    );
                 }
-                MessageModel msg = CreateMessage(
-                    status.CreatedDate,
-                    status.User,
-                    text
-                );
-                Session.AddMessageToChat(f_FriendsTimelineChat, msg);
 
                 f_LastFriendsTimelineStatusID = status.Id;
             }
@@ -1019,8 +1334,14 @@ namespace Smuxi.Engine
 #if LOG4NET
                 f_Logger.Error("UpdateRepliesThread(): Exception", ex);
 #endif
-                string msg =_("An error occurred while fetching the replies from Twitter. Reason: ");
-                Session.AddTextToChat(f_ProtocolChat, "-!- " + msg + ex.Message);
+                var msg = CreateMessageBuilder().
+                    AppendEventPrefix().
+                    AppendErrorText(
+                        _("An error occurred while fetching the replies " +
+                          "from Twitter. Reason: {0}"),
+                        ex.Message).
+                    ToMessage();
+                Session.AddMessageToChat(Chat, msg);
             } finally {
 #if LOG4NET
                 f_Logger.Debug("UpdateRepliesThread(): finishing thread.");
@@ -1061,14 +1382,18 @@ namespace Smuxi.Engine
             bool highlight = f_LastReplyStatusID != 0;
             List<TwitterStatus> sortedTimeline = SortTimeline(timeline);
             foreach (TwitterStatus status in sortedTimeline) {
-                MessageModel msg = CreateMessage(
-                    status.CreatedDate,
-                    status.User,
-                    status.Text,
-                    highlight
-                );
+                AddIndexToStatus(status);
+                var msg = CreateMessageBuilder().
+                    Append(status, GetPerson(status.User), highlight).
+                    ToMessage();
                 Session.AddMessageToChat(f_RepliesChat, msg);
 
+                OnMessageReceived(
+                    new MessageEventArgs(f_RepliesChat, msg,
+                                         status.User.ScreenName,
+                                         status.InReplyToScreenName ?? String.Empty)
+                );
+
                 f_LastReplyStatusID = status.Id;
             }
         }
@@ -1115,8 +1440,14 @@ namespace Smuxi.Engine
 #if LOG4NET
                 f_Logger.Error("UpdateDirectMessagesThread(): Exception", ex);
 #endif
-                string msg =_("An error occurred while fetching direct messages from Twitter. Reason: ");
-                Session.AddTextToChat(f_ProtocolChat, "-!- " + msg + ex.Message);
+                var msg = CreateMessageBuilder().
+                    AppendEventPrefix().
+                    AppendErrorText(
+                        _("An error occurred while fetching direct messages " +
+                          "from Twitter. Reason: {0}"),
+                        ex.Message).
+                    ToMessage();
+                Session.AddMessageToChat(Chat, msg);
             } finally {
 #if LOG4NET
                 f_Logger.Debug("UpdateDirectMessagesThread(): finishing thread.");
@@ -1197,12 +1528,9 @@ namespace Smuxi.Engine
                 // this is a new one!
                 bool highlight = receivedTimeline.Contains(directMsg) &&
                                  f_LastDirectMessageReceivedStatusID != 0;
-                MessageModel msg = CreateMessage(
-                    directMsg.CreatedDate,
-                    directMsg.Sender,
-                    directMsg.Text,
-                    highlight
-                );
+                var msg = CreateMessageBuilder().
+                    Append(directMsg, GetPerson(directMsg.Sender), highlight).
+                    ToMessage();
                 Session.AddMessageToChat(f_DirectMessagesChat, msg);
 
                 // if there is a tab open for this user put the message there too
@@ -1210,9 +1538,19 @@ namespace Smuxi.Engine
                 if (receivedTimeline.Contains(directMsg)) {
                     // this is a received message
                     userId =  directMsg.SenderId.ToString();
+
+                    OnMessageReceived(
+                        new MessageEventArgs(f_DirectMessagesChat, msg,
+                                             directMsg.SenderScreenName, null)
+                    );
                 } else {
                     // this is a sent message
                     userId = directMsg.RecipientId.ToString();
+
+                    OnMessageSent(
+                        new MessageEventArgs(f_DirectMessagesChat, msg,
+                                             null, directMsg.RecipientScreenName)
+                    );
                 }
                 ChatModel chat =  Session.GetChat(
                     userId,
@@ -1302,33 +1640,15 @@ namespace Smuxi.Engine
             CheckResponse(response);
             var user = response.ResponseObject;
             f_TwitterUser = user;
+            Me = CreatePerson(f_TwitterUser);
 #if LOG4NET
             f_Logger.Debug("UpdateUser(): done.");
 #endif
         }
 
-        private MessageModel CreateMessage(DateTime when, TwitterUser from,
-                                           string message)
+        protected new TwitterMessageBuilder CreateMessageBuilder()
         {
-            return CreateMessage(when, from, message, false);
-        }
-
-        private MessageModel CreateMessage(DateTime when, TwitterUser from,
-                                           string message, bool highlight)
-        {
-            if (from == null) {
-                throw new ArgumentNullException("from");
-            }
-            if (message == null) {
-                throw new ArgumentNullException("message");
-            }
-
-            var builder = CreateMessageBuilder();
-            // MessageModel serializer expects UTC values
-            builder.TimeStamp = when.ToUniversalTime();
-            builder.AppendSenderPrefix(GetPerson(from), highlight);
-            builder.AppendMessage(message);
-            return builder.ToMessage();
+            return CreateMessageBuilder<TwitterMessageBuilder>();
         }
 
         private T CreateOptions<T>() where T : OptionalProperties, new()
@@ -1339,9 +1659,16 @@ namespace Smuxi.Engine
             return options;
         }
 
-        private void PostUpdate(string text)
+        void PostUpdate(string text)
         {
-            var options = CreateOptions<StatusUpdateOptions>();
+            PostUpdate(text, null);
+        }
+
+        void PostUpdate(string text, StatusUpdateOptions options)
+        {
+            if (options == null) {
+                options = CreateOptions<StatusUpdateOptions>();
+            }
             var res = TwitterStatus.Update(f_OAuthTokens, text, options);
             CheckResponse(res);
             f_FriendsTimelineEvent.Set();
@@ -1354,7 +1681,35 @@ namespace Smuxi.Engine
             CheckResponse(res);
             f_DirectMessageEvent.Set();
         }
-        
+
+        void AddIndexToStatus(TwitterStatus status)
+        {
+            lock (StatusIndex) {
+                var slot = ++StatusIndexOffset;
+                if (slot > StatusIndex.Length) {
+                    StatusIndexOffset = 0;
+                    slot = 1;
+                }
+                StatusIndex[slot - 1] = status;
+                status.Text = String.Format("[{0:00}] {1}", slot, status.Text);
+                var rtStatus = status.RetweetedStatus;
+                if (rtStatus != null) {
+                    rtStatus.Text = String.Format("[{0:00}] {1}", slot,
+                                                  rtStatus.Text);
+                }
+            }
+        }
+
+        TwitterStatus GetStatusFromIndex(int slot)
+        {
+            lock (StatusIndex) {
+                if (slot > StatusIndex.Length || slot < 1) {
+                    return null;
+                }
+                return StatusIndex[slot - 1];
+            }
+        }
+
         private void CheckTwitterizerException(TwitterizerException exception)
         {
             Trace.Call(exception == null ? null : exception.GetType());
