@@ -1,6 +1,6 @@
 // Smuxi - Smart MUltipleXed Irc
 // 
-// Copyright (c) 2010-2012 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2010-2013 Mirco Bauer <meebey@meebey.net>
 // Copyright (c) 2013 Oliver Schneider <mail@oli-obk.de>
 // 
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
@@ -26,6 +26,7 @@ using System.Collections.Generic;
 using System.IO;
 using System.Xml;
 using System.Web;
+using Smuxi.Common;
 
 namespace Smuxi.Engine
 {
@@ -34,6 +35,7 @@ namespace Smuxi.Engine
 #if LOG4NET
         private static readonly log4net.ILog f_Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
+        static readonly string LibraryTextDomain = "smuxi-engine";
         MessageModel Message { get; set; }
         public bool NickColors { get; set; }
         public bool StripFormattings { get; set; }
@@ -614,7 +616,14 @@ namespace Smuxi.Engine
             }
             if (node.HasChildNodes) {
                 foreach (XmlNode child in node.ChildNodes) {
-                    ParseHtml(child, submodel);
+                    // clone this model
+                    TextMessagePartModel nextmodel;
+                    if (submodel is UrlMessagePartModel) {
+                        nextmodel = new UrlMessagePartModel(submodel);
+                    } else {
+                        nextmodel = new TextMessagePartModel(submodel);
+                    }
+                    ParseHtml(child, nextmodel);
                 }
             } else {
                 // final node
@@ -623,8 +632,7 @@ namespace Smuxi.Engine
                 } else if (nodetype == "img") {
                     AppendUrl(node.Attributes.GetNamedItem("src").Value, "[image placeholder - UNIMPLEMENTED]");
                 } else {
-                    model.Text = node.Value.Replace("\r", "").Replace("\n", "");
-                    model.Text = HttpUtility.HtmlDecode(model.Text);
+                    model.Text = HttpUtility.HtmlDecode(node.Value);
                     AppendText(model);
                 }
             }
@@ -632,6 +640,7 @@ namespace Smuxi.Engine
 
         public virtual MessageBuilder AppendHtmlMessage(string html)
         {
+            html = NormalizeNewlines(html);
             XmlDocument doc = new XmlDocument();
             try {
                 // wrap in div to prevent messages beginning with text from failing "to be xml"
@@ -763,5 +772,87 @@ namespace Smuxi.Engine
             }
             return this;
         }
+
+        public virtual MessageBuilder AppendChatState(ContactModel contact, MessageType state)
+        {
+            switch (state) {
+                case MessageType.ChatStateComposing:
+                    if (Message.IsEmpty) {
+                        AppendActionPrefix();
+                        AppendFormat(_("{0} is typing..."), contact);
+                    }
+                    break;
+                case MessageType.ChatStatePaused:
+                    if (Message.IsEmpty) {
+                        AppendActionPrefix();
+                        AppendFormat(_("{0} has stopped typing..."), contact);
+                    }
+                    break;
+                case MessageType.ChatStateReset:
+                    break;
+                default:
+                    throw new ArgumentException("state is not a ChatState", "state");
+            }
+            MessageType = state;
+            return this;
+        }
+
+        protected static string NormalizeNewlines(string text)
+        {
+            if (text == null) {
+                throw new ArgumentNullException("text");
+            }
+            if (!text.Contains("\n")) {
+                // nothing to normalize
+                return text;
+            }
+
+            var normalized = new StringBuilder(text.Length);
+            text = text.Replace("\r\n", "\n");
+            foreach (var textPart in text.Split('\n')) {
+                var trimmed = textPart.TrimEnd(' ');
+                if (trimmed.Length == 0) {
+                    // skip empty lines
+                    continue;
+                }
+                normalized.AppendFormat("{0} ", trimmed);
+            }
+            // remove trailing space
+            normalized.Length--;
+            return normalized.ToString();
+        }
+
+        public virtual MessageBuilder AppendPresenceState(ContactModel contact, MessageType state)
+        {
+            switch (state) {
+                case MessageType.PresenceStateAway:
+                    if (Message.IsEmpty) {
+                        AppendActionPrefix();
+                        AppendFormat(_("{0} is away"), contact);
+                    }
+                    break;
+                case MessageType.PresenceStateOffline:
+                    if (Message.IsEmpty) {
+                        AppendActionPrefix();
+                        AppendFormat(_("{0} is offline"), contact);
+                    }
+                    break;
+                case MessageType.PresenceStateOnline:
+                    if (Message.IsEmpty) {
+                        AppendActionPrefix();
+                        AppendFormat(_("{0} is online"), contact);
+                    }
+                    break;
+                default:
+                    throw new ArgumentException("state is not a PresenceState", "state");
+            }
+            MessageType = state;
+            return this;
+        }
+
+        static string _(string msg)
+        {
+            return LibraryCatalog.GetString(msg, LibraryTextDomain);
+        }
     }
 }
