@@ -64,6 +64,18 @@ namespace Smuxi.Engine
                 return _ProtocolManagers;
             }
         }
+
+        public IProtocolManager FirstProtocolManager {
+            get {
+                lock (_ProtocolManagers) {
+                    if (_ProtocolManagers.Count == 0) {
+                        return null;
+                    } else {
+                        return _ProtocolManagers[0];
+                    }
+                }
+            }
+        }
         
         public IList<ChatModel> Chats {
             get {
@@ -149,6 +161,25 @@ namespace Smuxi.Engine
                                       TimeSpan.Zero, NewsFeedUpdateInterval);
         }
 
+        public IProtocolManager NextProtocolManager(IProtocolManager currentProtocolManager)
+        {
+            lock (_ProtocolManagers) {
+                if (_ProtocolManagers.Count == 0) {
+                    return null;
+                }
+                int pos = 0;
+                if (currentProtocolManager != null) {
+                    pos = _ProtocolManagers.IndexOf(currentProtocolManager);
+                }
+                if (pos < _ProtocolManagers.Count - 1) {
+                    pos++;
+                } else {
+                    pos = 0;
+                }
+                return _ProtocolManagers[pos];
+            }
+        }
+
         protected MessageBuilder CreateMessageBuilder()
         {
             var builder = new MessageBuilder();
@@ -417,6 +448,7 @@ namespace Smuxi.Engine
             } else {
                 // normal text
                 if (cd.Chat.ChatType == ChatType.Session &&
+                    cd.FrontendManager != null &&
                     cd.FrontendManager.CurrentProtocolManager == null) {
                     _NotConnected(cd);
                     handled = true;
@@ -437,7 +469,7 @@ namespace Smuxi.Engine
             // TRANSLATOR: this line is used as a label / category for a
             // list of commands below
             builder.AppendHeader(_("Engine Commands"));
-            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
+            AddMessageToFrontend(cd, builder.ToMessage());
 
             string[] help = {
                 "connect/server protocol [protocol-parameters]",
@@ -456,7 +488,7 @@ namespace Smuxi.Engine
                 builder = CreateMessageBuilder();
                 builder.AppendEventPrefix();
                 builder.AppendText(line);
-                cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
+                AddMessageToFrontend(cd, builder.ToMessage());
             }
         }
         
@@ -514,7 +546,9 @@ namespace Smuxi.Engine
             if (protocolManager == null && server == null) {
                 try {
                     protocolManager = CreateProtocolManager(protocol);
-                    _ProtocolManagers.Add(protocolManager);
+                    lock (_ProtocolManagers) {
+                        _ProtocolManagers.Add(protocolManager);
+                    }
                 } catch (ArgumentException ex) {
                     if (ex.ParamName != "protocol") {
                         throw;
@@ -523,7 +557,7 @@ namespace Smuxi.Engine
                     builder = CreateMessageBuilder();
                     builder.AppendEventPrefix();
                     builder.AppendErrorText(ex.Message);
-                    fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+                    AddMessageToFrontend(cd, builder.ToMessage());
                     return;
                 }
             }
@@ -588,7 +622,7 @@ namespace Smuxi.Engine
                         _("Disconnect failed - could not find server: {0}"),
                         server
                     );
-                    fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+                    AddMessageToFrontend(cd, builder.ToMessage());
                     return;
                 }
             } else {
@@ -600,7 +634,9 @@ namespace Smuxi.Engine
             }
             victim.Disconnect(fm);
             victim.Dispose();
-            _ProtocolManagers.Remove(victim);
+            lock (_ProtocolManagers) {
+                _ProtocolManagers.Remove(victim);
+            }
         }
         
         public void CommandReconnect(CommandModel cd)
@@ -626,8 +662,7 @@ namespace Smuxi.Engine
                     var builder = CreateMessageBuilder();
                     builder.AppendEventPrefix();
                     builder.AppendErrorText(_("Reconnect failed!"));
-                    cd.FrontendManager.AddMessageToChat(cd.Chat,
-                                                        builder.ToMessage());
+                    AddMessageToFrontend(cd, builder.ToMessage());
                 }
             });
         }
@@ -640,7 +675,6 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("cd");
             }
             
-            FrontendManager fm = cd.FrontendManager;
             if (cd.DataArray.Length < 2) {
                 _NotEnoughParameters(cd);
                 return;
@@ -675,7 +709,7 @@ namespace Smuxi.Engine
                         builder = CreateMessageBuilder();
                         builder.AppendEventPrefix();
                         builder.AppendText("{0} = {1}", entry.Key, entry.Value);
-                        fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+                        AddMessageToFrontend(cd, builder.ToMessage());
                     }
                     return;
                 case "set":
@@ -688,7 +722,7 @@ namespace Smuxi.Engine
                         builder.AppendErrorText(
                             _("Invalid key/value format.")
                         );
-                        fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+                        AddMessageToFrontend(cd, builder.ToMessage());
                         return;
                     }
                     string setKey = setParam.Split('=')[0];
@@ -725,24 +759,21 @@ namespace Smuxi.Engine
                     );
                     break;
             }
-            fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+            AddMessageToFrontend(cd, builder.ToMessage());
         }
         
         public void CommandShutdown(CommandModel cmd)
         {
             Trace.Call(cmd);
 
-            if (cmd == null) {
-                throw new ArgumentNullException("cmd");
-            }
-
+            FrontendManager frontendMgr = cmd != null ? cmd.FrontendManager : null;
 #if LOG4NET
             f_Logger.Info("Shutting down...");
 #endif
             lock (_ProtocolManagers) {
                 foreach (var protocolManager in _ProtocolManagers) {
                     try {
-                        protocolManager.Disconnect(cmd.FrontendManager);
+                        protocolManager.Disconnect(frontendMgr);
                         protocolManager.Dispose();
                     } catch (Exception ex) {
 #if LOG4NET
@@ -801,7 +832,6 @@ namespace Smuxi.Engine
                 return;
             }
 
-            FrontendManager fm = cd.FrontendManager;
             if (cd.DataArray.Length >= 2) {
                 switch (cd.DataArray[1].ToLower()) {
                     case "list":
@@ -820,7 +850,7 @@ namespace Smuxi.Engine
                             _("Invalid parameter for network; use list, " +
                               "switch, or close")
                         );
-                        fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+                        AddMessageToFrontend(cd, builder.ToMessage());
                         break;
                 }
             } else {
@@ -830,7 +860,6 @@ namespace Smuxi.Engine
         
         private void _CommandNetworkList(CommandModel cmd)
         {
-            var frontend = cmd.FrontendManager;
             var servers = new ServerListController(UserConfig);
             var availableNetworks = servers.GetNetworks();
             var connectedNetworks = new List<IProtocolManager>();
@@ -849,7 +878,7 @@ namespace Smuxi.Engine
             // TRANSLATOR: this line is used as a label / category for a
             // list of networks below
             msg.AppendHeader(_("Connected Networks"));
-            frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+            AddMessageToFrontend(cmd, msg.ToMessage());
             foreach (var network in connectedNetworks) {
                 msg = CreateMessageBuilder();
                 msg.AppendEventPrefix();
@@ -857,20 +886,20 @@ namespace Smuxi.Engine
                 msg.AppendText("{0}: {1} ", _("Protocol"), network.Protocol);
                 msg.AppendText("{0}: {1} ", _("Host"),     network.Host);
                 msg.AppendText("{0}: {1}",  _("Port"),     network.Port);
-                frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+                AddMessageToFrontend(cmd, msg.ToMessage());
             }
             if (connectedNetworks.Count == 0) {
                 msg = CreateMessageBuilder();
                 // TRANSLATOR: no connected networks
                 msg.AppendEventPrefix().AppendText("<{0}>", _("None"));
-                frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+                AddMessageToFrontend(cmd, msg.ToMessage());
             }
 
             msg = CreateMessageBuilder();
             // TRANSLATOR: this line is used as a label / category for a
             // list of networks below
             msg.AppendHeader(_("Available Networks"));
-            frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+            AddMessageToFrontend(cmd, msg.ToMessage());
             foreach (var network in availableNetworks) {
                 if (network == null || network.Trim().Length == 0) {
                     continue;
@@ -878,19 +907,18 @@ namespace Smuxi.Engine
                 msg = CreateMessageBuilder();
                 msg.AppendEventPrefix();
                 msg.AppendText("{0}: {1}", _("Network"), network);
-                frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+                AddMessageToFrontend(cmd, msg.ToMessage());
             }
             if (availableNetworks.Count == 0) {
                 msg = CreateMessageBuilder();
                 // TRANSLATOR: no available networks
                 msg.AppendEventPrefix().AppendText("<{0}>", _("None"));
-                frontend.AddMessageToChat(cmd.Chat, msg.ToMessage());
+                AddMessageToFrontend(cmd, msg.ToMessage());
             }
         }
         
         private void _CommandNetworkClose(CommandModel cd)
         {
-            FrontendManager fm = cd.FrontendManager;
             IProtocolManager pm = null;
             if (cd.DataArray.Length >= 3) {
                 // named protocol manager
@@ -900,7 +928,7 @@ namespace Smuxi.Engine
                     var builder = CreateMessageBuilder();
                     builder.AppendText(_("Network close failed - could not " +
                                          "find network: {0}"), network);
-                    fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+                    AddMessageToFrontend(cd, builder.ToMessage());
                     return;
                 }
             } else if (cd.DataArray.Length >= 2) {
@@ -915,10 +943,12 @@ namespace Smuxi.Engine
             // disconnect in background as could be blocking
             ThreadPool.QueueUserWorkItem(delegate {
                 try {
-                    pm.Disconnect(fm);
+                    pm.Disconnect(cd.FrontendManager);
                     pm.Dispose();
                     // Dispose() takes care of removing the chat from session (frontends)
-                    _ProtocolManagers.Remove(pm);
+                    lock (_ProtocolManagers) {
+                        _ProtocolManagers.Remove(pm);
+                    }
                 } catch (Exception ex) {
 #if LOG4NET
                     f_Logger.Error("_CommandNetworkClose(): Exception", ex);
@@ -930,6 +960,10 @@ namespace Smuxi.Engine
         private void _CommandNetworkSwitch(CommandModel cd)
         {
             FrontendManager fm = cd.FrontendManager;
+            if (fm == null) {
+                _NotEnoughParameters(cd);
+                return;
+            }
             if (cd.DataArray.Length >= 3) {
                 // named network manager
                 string network = cd.DataArray[2];
@@ -955,7 +989,7 @@ namespace Smuxi.Engine
         {
             var builder = CreateMessageBuilder();
             builder.AppendText(_("Not connected to any network"));
-            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
+            AddMessageToFrontend(cd, builder.ToMessage());
         }
         
         private void _NotEnoughParameters(CommandModel cd)
@@ -963,7 +997,7 @@ namespace Smuxi.Engine
             var builder = CreateMessageBuilder();
             builder.AppendText(_("Not enough parameters for {0} command"),
                                cd.Command);
-            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
+            AddMessageToFrontend(cd, builder.ToMessage());
         }
         
         public void UpdateNetworkStatus()
@@ -1370,13 +1404,41 @@ namespace Smuxi.Engine
             IProtocolManager protocolManager = CreateProtocolManager(
                 server.Protocol
             );
-            _ProtocolManagers.Add(protocolManager);
+            lock (_ProtocolManagers) {
+                _ProtocolManagers.Add(protocolManager);
+            }
 
             string password = null;
             // only pass non-empty passwords to Connect()
             if (!String.IsNullOrEmpty(server.Password)) {
                 password = server.Password;
             }
+
+            if (server.OnConnectCommands != null && server.OnConnectCommands.Count > 0) {
+                protocolManager.Connected += delegate {
+                    ThreadPool.QueueUserWorkItem(delegate {
+                        try {
+                            foreach (string command in server.OnConnectCommands) {
+                                if (command.Length == 0) {
+                                    continue;
+                                }
+                                CommandModel cd = new CommandModel(
+                                    frontendManager,
+                                    protocolManager.Chat,
+                                    (string) _UserConfig["Interface/Entry/CommandCharacter"],
+                                    command
+                                );
+                                protocolManager.Command(cd);
+                            }
+                        } catch (Exception ex) {
+#if LOG4NET
+                            f_Logger.Error("Connected event: Exception", ex);
+#endif
+                        }
+                    });
+                };
+            }
+
             protocolManager.Connect(frontendManager, server);
             if (protocolManager.Chat == null) {
                 // just in case the ProtocolManager is not setting the
@@ -1384,23 +1446,6 @@ namespace Smuxi.Engine
                 throw new ApplicationException(_("Connect failed."));
             }
 
-            if (server.OnConnectCommands != null && server.OnConnectCommands.Count > 0) {
-                protocolManager.Connected += delegate {
-                    foreach (string command in server.OnConnectCommands) {
-                        if (command.Length == 0) {
-                            continue;
-                        }
-                        CommandModel cd = new CommandModel(
-                            frontendManager,
-                            protocolManager.Chat,
-                            (string) _UserConfig["Interface/Entry/CommandCharacter"],
-                            command
-                        );
-                        protocolManager.Command(cd);
-                    }
-                };
-            }
-
             return protocolManager;
         }
         
@@ -1455,6 +1500,15 @@ namespace Smuxi.Engine
                 if (protocol == "twitter") {
                     return;
                 }
+
+                // don't log chatstates
+                switch (msg.MessageType) {
+                    case MessageType.ChatStateComposing:
+                    case MessageType.ChatStatePaused:
+                    case MessageType.ChatStateReset:
+                        return;
+                }
+
                 using (var stream = File.AppendText(chat.LogFile)) {
                     stream.WriteLine(
                         String.Format(
@@ -1530,6 +1584,32 @@ namespace Smuxi.Engine
             return false;
         }
 
+        public void AddMessageToFrontend(CommandModel cmd, MessageModel msg)
+        {
+            if (cmd == null) {
+                throw new ArgumentNullException("cmd");
+            }
+
+            AddMessageToFrontend(cmd.FrontendManager, cmd.Chat, msg);
+        }
+
+        public void AddMessageToFrontend(FrontendManager fm, ChatModel chat, MessageModel msg)
+        {
+            if (chat == null) {
+                throw new ArgumentNullException("chat");
+            }
+            if (msg == null) {
+                throw new ArgumentNullException("msg");
+            }
+
+            if (fm == null) {
+                // fallback to session
+                AddMessageToChat(chat, msg);
+            } else {
+                fm.AddMessageToChat(chat, msg);
+            }
+        }
+
        void OnUserConfigChanged(object sender, ConfigChangedEventArgs e)
        {
             if (e.Key.StartsWith("Filters/")) {
