@@ -598,7 +598,13 @@ namespace Smuxi.Frontend.Gnome
                 });
                 return;
             }
-            
+
+            if (ex is NotImplementedException) {
+                // don't quit on NotImplementedException
+                ShowError(parent, ex);
+                return;
+            }
+
 #if LOG4NET
             _Logger.Error("ShowException(): Exception:", ex);
 #endif
@@ -649,7 +655,8 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call(engine);
             
-            EngineManagerDialog diag = new EngineManagerDialog(_MainWindow.EngineManager);
+            var diag = new EngineManagerDialog(_MainWindow,
+                                               _MainWindow.EngineManager);
             if (!String.IsNullOrEmpty(engine)) {
                 diag.SelectedEngine = engine;
                 // 1 == connect button
