@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2011 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -48,6 +48,7 @@ namespace Smuxi.Frontend.Gnome
         public IFrontendUI UI { get; private set; }
         public Entry Entry { get; private set; }
         public Notebook Notebook { get; private set; }
+        public ChatTreeView ChatTreeView { get; private set; }
         public ChatViewManager ChatViewManager { get; private set; }
         public EngineManager EngineManager { get; private set; }
 #if GTK_SHARP_2_10
@@ -182,12 +183,13 @@ namespace Smuxi.Frontend.Gnome
             FocusOutEvent += OnFocusOutEvent;
             WindowStateEvent += OnWindowStateEvent;
 
-            // TODO: network treeview
+            ChatTreeView = new ChatTreeView();
+
             Notebook = new Notebook();
             Notebook.SwitchPage += OnNotebookSwitchPage;
             Notebook.FocusInEvent += OnNotebookFocusInEvent;
 
-            ChatViewManager = new ChatViewManager(Notebook, null);
+            ChatViewManager = new ChatViewManager(Notebook, ChatTreeView);
             Assembly asm = Assembly.GetExecutingAssembly();
             ChatViewManager.Load(asm);
             ChatViewManager.LoadAll(System.IO.Path.GetDirectoryName(asm.Location),
@@ -247,23 +249,37 @@ namespace Smuxi.Frontend.Gnome
 
             MenuWidget = new MenuWidget(this, ChatViewManager);
 
-            Gtk.VPaned vpane = new Gtk.VPaned();
-            vpane.ButtonPressEvent += (sender, e) => {;
+            var treeviewScrolledWindow = new Gtk.ScrolledWindow() {
+                ShadowType = Gtk.ShadowType.EtchedIn,
+                HscrollbarPolicy = Gtk.PolicyType.Never,
+                VscrollbarPolicy = Gtk.PolicyType.Automatic
+            };
+            treeviewScrolledWindow.Add(ChatTreeView);
+            ChatViewManager.ChatAdded += (sender, e) => {
+                treeviewScrolledWindow.CheckResize();
+            };
+
+            var treeviewPaned = new Gtk.HPaned();
+            treeviewPaned.Pack1(treeviewScrolledWindow, false, false);
+            treeviewPaned.Pack2(Notebook, true, false);
+
+            var entryPaned = new Gtk.VPaned();
+            entryPaned.ButtonPressEvent += (sender, e) => {
                 // reset entry size on double click
                 if (e.Event.Type == Gdk.EventType.TwoButtonPress &&
                     e.Event.Button == 1) {
                     GLib.Timeout.Add(100, delegate {
-                        vpane.Position = -1;
+                        entryPaned.Position = -1;
                         return false;
                     });
                 }
             };
-            vpane.Pack1(Notebook, true, false);
-            vpane.Pack2(entryScrolledWindow, false, false);
+            entryPaned.Pack1(treeviewPaned, true, false);
+            entryPaned.Pack2(entryScrolledWindow, false, false);
 
             Gtk.VBox vbox = new Gtk.VBox();
             vbox.PackStart(MenuWidget, false, false, 0);
-            vbox.PackStart(vpane, true, true, 0);
+            vbox.PackStart(entryPaned, true, true, 0);
 
             NetworkStatusbar = new Gtk.Statusbar();
             NetworkStatusbar.WidthRequest = 300;
@@ -303,6 +319,7 @@ namespace Smuxi.Frontend.Gnome
             NotificationAreaIconMode = mode;
 
             MenuWidget.OpenLogAction.Visible = Frontend.IsLocalEngine;
+            MenuWidget.OpenLogToolAction.Visible = Frontend.IsLocalEngine;
 
 #if GTK_SHARP_2_10
             StatusIconManager.ApplyConfig(userConfig);
@@ -315,6 +332,7 @@ namespace Smuxi.Frontend.Gnome
 #endif
             Entry.ApplyConfig(userConfig);
             Notebook.ApplyConfig(userConfig);
+            ChatTreeView.ApplyConfig(userConfig);
             ChatViewManager.ApplyConfig(userConfig);
             MenuWidget.JoinWidget.ApplyConfig(userConfig);
         }
@@ -329,7 +347,7 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(chatView, protocolStatus);
 
             if (chatView == null) {
-                chatView = Notebook.CurrentChatView;
+                chatView = ChatViewManager.CurrentChatView;
             }
             if (chatView == null) {
                 return;
@@ -396,11 +414,12 @@ namespace Smuxi.Frontend.Gnome
                     return;
                 }
 
-                ChatView chatView = Notebook.CurrentChatView;
+                var chatView = ChatViewManager.CurrentChatView;
                 if (chatView != null) {
                     // clear activity and highlight
                     chatView.HasHighlight = false;
                     chatView.HasActivity = false;
+                    chatView.HasEvent = false;
                     var lastMsg = chatView.OutputMessageTextView.LastMessage;
                     if (lastMsg == null || Frontend.UseLowBandwidthMode) {
                         return;
@@ -431,7 +450,7 @@ namespace Smuxi.Frontend.Gnome
                     return;
                 }
 
-                var chatView = Notebook.CurrentChatView;
+                var chatView = ChatViewManager.CurrentChatView;
                 if (chatView == null) {
                     return;
                 }
@@ -487,7 +506,7 @@ namespace Smuxi.Frontend.Gnome
         protected virtual void OnNotebookSwitchPage(object sender, EventArgs e)
         {
             try {
-                var chatView = Notebook.CurrentChatView;
+                var chatView = ChatViewManager.CurrentChatView;
                 if (chatView == null) {
                     return;
                 }
@@ -497,8 +516,9 @@ namespace Smuxi.Frontend.Gnome
                 }
                 MenuWidget.FindGroupChatAction.Sensitive = !(chatView is SessionChatView);
                 if (Frontend.IsLocalEngine) {
-                    MenuWidget.OpenLogAction.Sensitive =
-                        File.Exists(chatView.ChatModel.LogFile);
+                    var logExists = File.Exists(chatView.ChatModel.LogFile);
+                    MenuWidget.OpenLogAction.Sensitive = logExists;
+                    MenuWidget.OpenLogToolAction.Sensitive = logExists;
                 }
 
                 // find protocol chat parent and update join bar
@@ -516,10 +536,12 @@ namespace Smuxi.Frontend.Gnome
 
                 // HACK: Gtk.Notebook moves the focus to the child after the
                 // page has been switched, so move the focus back to the entry
-                GLib.Idle.Add(delegate {
-                    Entry.GrabFocus();
-                    return false;
-                });
+                if (!Notebook.IsBrowseModeEnabled) {
+                    GLib.Idle.Add(delegate {
+                        Entry.GrabFocus();
+                        return false;
+                    });
+                }
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
@@ -537,6 +559,9 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
 
             e.ChatView.MessageHighlighted += OnChatViewMessageHighlighted;
+            e.ChatView.StatusChanged += (o, args) => {
+                ChatTreeView.Render(e.ChatView);
+            };
             e.ChatView.OutputMessageTextView.FocusInEvent += delegate {
                 if (CaretMode) {
                     return;
@@ -558,6 +583,8 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
             
             e.ChatView.MessageHighlighted -= OnChatViewMessageHighlighted;
+
+            UpdateProgressBar();
         }
         
         protected void OnChatViewMessageHighlighted(object sender, ChatViewMessageHighlightedEventArgs e)
