@@ -40,7 +40,7 @@ namespace Smuxi.Frontend.Gnome
         public    int                Position { get; internal set; }
         private   ChatModel          _ChatModel;
         private   bool               _HasHighlight;
-        private   int                HighlightCount { get; set; }
+        public    int                HighlightCount { get; private set; }
         private   bool               _HasActivity;
         private   bool               _HasEvent;
         private   bool               _IsSynced;
@@ -58,10 +58,12 @@ namespace Smuxi.Frontend.Gnome
         protected string             SyncedName { get; set; }
         public    IProtocolManager   ProtocolManager { get; set; }
         bool                         UseLowBandwidthMode { get; set; }
-        protected Gtk.Image          TabImage { get; set; }
+        public Gtk.Image TabImage { get; protected set; }
         bool                         IsAutoScrolling { get; set; }
         Gtk.ImageMenuItem  CloseItem { get; set; }
 
+        public event EventHandler<EventArgs> StatusChanged;
+
         public ChatModel ChatModel {
             get {
                 return _ChatModel;
@@ -75,6 +77,8 @@ namespace Smuxi.Frontend.Gnome
             set {
                 base.Name = value;
                 _TabLabel.Text = value;
+
+                OnStatusChanged(EventArgs.Empty);
             }
         }
 
@@ -85,7 +89,7 @@ namespace Smuxi.Frontend.Gnome
                 // object?!?
                 return Frontend.MainWindow.HasToplevelFocus &&
                         Object.ReferenceEquals(
-                            Frontend.MainWindow.Notebook.CurrentChatView,
+                            Frontend.MainWindow.ChatViewManager.CurrentChatView,
                             this
                         );
             }
@@ -96,13 +100,20 @@ namespace Smuxi.Frontend.Gnome
                 return _HasHighlight;
             }
             set {
-                _HasHighlight = value;
-                HighlightCount++;
-
-                if (!value) {
+                if (value) {
+                    _HasHighlight = value;
+                    HighlightCount++;
+                    OnStatusChanged(EventArgs.Empty);
+                } else {
+                    if (_HasHighlight == value) {
+                        // nothing to update
+                        return;
+                    }
+                    _HasHighlight = value;
                     // clear highlight with "no activity"
                     HasActivity = false;
                     HighlightCount = 0;
+                    OnStatusChanged(EventArgs.Empty);
                     return;
                 }
 
@@ -136,11 +147,12 @@ namespace Smuxi.Frontend.Gnome
                 return _HasActivity;
             }
             set {
-                if (value && _HasActivity == value) {
+                if (_HasActivity == value) {
                     // nothing to update
                     return;
                 }
                 _HasActivity = value;
+                OnStatusChanged(EventArgs.Empty);
 
                 if (HasHighlight) {
                     // don't show activity if there is a highlight active
@@ -172,7 +184,12 @@ namespace Smuxi.Frontend.Gnome
                 return _HasEvent;
             }
             set {
+                if (_HasEvent == value) {
+                    // nothing to update
+                    return;
+                }
                 _HasEvent = value;
+                OnStatusChanged(EventArgs.Empty);
 
                 if (HasHighlight) {
                     return;
@@ -249,7 +266,7 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
-        protected Gtk.Menu TabMenu {
+        public Gtk.Menu TabMenu {
             get {
                 return _TabMenu;
             }
@@ -332,6 +349,10 @@ namespace Smuxi.Frontend.Gnome
             _LastSeenHighlightQueue.AbortedEvent += OnLastSeenHighlightQueueAbortedEvent;
             _LastSeenHighlightQueue.ExceptionEvent += OnLastSeenHighlightQueueExceptionEvent;
         }
+
+        protected ChatView(IntPtr handle) : base(handle)
+        {
+        }
         
         ~ChatView()
         {
@@ -481,6 +502,7 @@ namespace Smuxi.Frontend.Gnome
 
             GLib.Idle.Add(delegate {
                 TabImage.SetFromStock(Gtk.Stock.Refresh, Gtk.IconSize.Menu);
+                OnStatusChanged(EventArgs.Empty);
                 return false;
             });
 
@@ -539,6 +561,7 @@ namespace Smuxi.Frontend.Gnome
             // show all chats with message activity after the frontend connect
             if (!HasHighlight) {
                 HasActivity = false;
+                HasEvent = false;
             }
 
             // let the user know at which position new messages start
@@ -546,6 +569,7 @@ namespace Smuxi.Frontend.Gnome
 
             // reset tab icon to normal
             TabImage.Pixbuf = DefaultTabImage.Pixbuf;
+            OnStatusChanged(EventArgs.Empty);
 
             SyncedMessages = null;
             _IsSynced = true;
@@ -553,6 +577,21 @@ namespace Smuxi.Frontend.Gnome
         
         public virtual void AddMessage(MessageModel msg)
         {
+            switch (msg.MessageType) {
+                case MessageType.ChatNameChanged:
+                    ThreadPool.QueueUserWorkItem(delegate {
+                        try {
+                            // REMOTING CALL
+                            var newname = ChatModel.Name;
+                            Gtk.Application.Invoke(delegate {
+                                Name = newname;
+                            });
+                        } catch (Exception ex) {
+                            Frontend.ShowException(ex);
+                        }
+                    });
+                    return;
+            }
             _OutputMessageTextView.AddMessage(msg);
         }
         
@@ -638,9 +677,7 @@ namespace Smuxi.Frontend.Gnome
         
         protected virtual void OnMessageTextViewMessageAdded(object sender, MessageTextViewMessageAddedEventArgs e)
         {
-            // HACK: out of scope?
-            // probably we should use the ChatViewManager instead?
-            if (Frontend.MainWindow.Notebook.CurrentChatView != this) {
+            if (!IsActive) {
                 switch (e.Message.MessageType) {
                     case MessageType.Normal:
                         HasActivity = true;
@@ -803,6 +840,13 @@ namespace Smuxi.Frontend.Gnome
 #endif
         }
 
+        protected virtual void OnStatusChanged(EventArgs e)
+        {
+            if (StatusChanged != null) {
+                StatusChanged(this, e);
+            }
+        }
+
         void OnVadjustmentValueChanged(object sender, EventArgs e)
         {
             CheckAutoScroll();
