@@ -137,6 +137,8 @@ namespace Smuxi.Frontend.Gnome
                 }
             };
 
+            Gtk.TreeView tv = new Gtk.TreeView();
+            _PersonTreeView = tv;
             Gtk.ScrolledWindow sw = new Gtk.ScrolledWindow();
             PersonScrolledWindow = sw;
             sw.HscrollbarPolicy = Gtk.PolicyType.Never;
@@ -164,8 +166,6 @@ namespace Smuxi.Frontend.Gnome
                 args.Requisition = bestSize;
             };
 
-            Gtk.TreeView tv = new Gtk.TreeView();
-            _PersonTreeView = tv;
             //tv.CanFocus = false;
             tv.BorderWidth = 0;
             tv.Selection.Mode = Gtk.SelectionMode.Multiple;
@@ -236,9 +236,9 @@ namespace Smuxi.Frontend.Gnome
             _TopicScrolledWindow.NoShowAll = true;
             _TopicScrolledWindow.SizeRequested += delegate(object o, Gtk.SizeRequestedArgs args) {
                 // predict and set useful topic heigth
-                int lineWidth, lineHeigth;
+                int lineWidth, lineHeight;
                 using (var layout = _TopicTextView.CreatePangoLayout("Test Topic")) {
-                    layout.GetPixelSize(out lineWidth, out lineHeigth);
+                    layout.GetPixelSize(out lineWidth, out lineHeight);
                 }
                 var lineSpacing = _TopicTextView.PixelsAboveLines +
                                   _TopicTextView.PixelsBelowLines;
@@ -253,7 +253,7 @@ namespace Smuxi.Frontend.Gnome
                 }
                 newLines = Math.Min(newLines, 3);
                 var bestSize = new Gtk.Requisition() {
-                    Height = ((lineHeigth + lineSpacing) * newLines) + 2
+                    Height = ((lineHeight + lineSpacing) * newLines) + 4
                 };
                 args.Requisition = bestSize;
             };
@@ -265,6 +265,10 @@ namespace Smuxi.Frontend.Gnome
             ShowAll();
         }
 
+        protected GroupChatView(IntPtr handle) : base(handle)
+        {
+        }
+
         public override void Dispose()
         {
             Trace.Call();
@@ -299,6 +303,7 @@ namespace Smuxi.Frontend.Gnome
 
             GLib.Idle.Add(delegate {
                 TabImage.SetFromStock(Gtk.Stock.Refresh, Gtk.IconSize.Menu);
+                OnStatusChanged(EventArgs.Empty);
                 return false;
             });
 
@@ -528,6 +533,9 @@ namespace Smuxi.Frontend.Gnome
             _PersonTreeView.ModifyFont(ThemeSettings.FontDescription);
             
             string userlist_pos = (string) config["Interface/Notebook/Channel/UserListPosition"];
+            if (userlist_pos == "left") {
+                userlist_pos = "right";
+            }
             if (_PersonTreeViewFrame.IsAncestor(_OutputHPaned)) {
                 _OutputHPaned.Remove(_PersonTreeViewFrame);
             }
