@@ -137,6 +137,10 @@ namespace Smuxi.Frontend.Gnome
             ShowAll();
         }
 
+        protected ProtocolChatView(IntPtr handle) : base(handle)
+        {
+        }
+
         public override void Sync()
         {
             Trace.Call();
@@ -361,6 +365,7 @@ namespace Smuxi.Frontend.Gnome
             ServerIconPixbuf = new Gdk.Pixbuf(iconPath, 16, 16);
             GLib.Idle.Add(delegate {
                 TabImage.Pixbuf = ServerIconPixbuf;
+                OnStatusChanged(EventArgs.Empty);
                 return false;
             });
         }
