@@ -61,6 +61,12 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        public Gtk.Action OpenLogToolAction {
+            get {
+                return f_OpenLogToolAction;
+            }
+        }
+
         public Gtk.Action CloseChatAction {
             get {
                 return f_CloseChatAction;
@@ -116,13 +122,14 @@ namespace Smuxi.Frontend.Gnome
 
             JoinWidget = new JoinWidget();
             JoinWidget.NoShowAll = true;
-            JoinWidget.Visible = f_MenuToolbar.Visible;
             JoinWidget.Activated += OnJoinWidgetActivated;
 
             var joinToolItem = new Gtk.ToolItem();
             joinToolItem.Add(JoinWidget);
             f_JoinToolbar.Add(joinToolItem);
             f_JoinToolbar.ShowAll();
+            f_JoinToolbar.NoShowAll = true;
+            f_JoinToolbar.Visible = f_MenuToolbar.Visible;
 
             f_ShowMenubarAction.Active = (bool) Frontend.FrontendConfig["ShowMenuBar"];
             f_ShowToolbarAction.Active = (bool) Frontend.FrontendConfig["ShowToolBar"];
@@ -446,7 +453,13 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
 
             try {
-                MainWindow.Notebook.IsBrowseModeEnabled = !MainWindow.Notebook.IsBrowseModeEnabled;
+                var browseMode = f_BrowseModeAction.Active;
+                MainWindow.Notebook.IsBrowseModeEnabled = browseMode;
+                if (browseMode) {
+                    MainWindow.ChatTreeView.HasFocus = true;
+                } else {
+                    MainWindow.Entry.HasFocus = true;
+                }
             } catch (Exception ex) {
                 Frontend.ShowException(Parent, ex);
             }
@@ -474,7 +487,7 @@ namespace Smuxi.Frontend.Gnome
                 var active = f_ShowToolbarAction.Active;
                 f_MenuToolbar.Visible = active;
                 // also hide/show join bar
-                JoinWidget.Visible = active;
+                f_JoinToolbar.Visible = active;
                 Frontend.FrontendConfig["ShowToolBar"] = active;
                 Frontend.FrontendConfig.Save();
             } catch (Exception ex) {
