@@ -20,6 +20,7 @@
 
 using System;
 using System.Linq;
+using System.Reflection;
 using System.Collections.Generic;
 using SysDiag = System.Diagnostics;
 using Smuxi.Common;
@@ -46,6 +47,7 @@ namespace Smuxi.Frontend
         static readonly log4net.ILog f_Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
         static readonly string f_LibraryTextDomain = "smuxi-frontend";
+        static string FrontendVersion { get; set; }
         Session         f_Session;
         TaskQueue       f_TaskQueue;
         TimeSpan        f_LastCommandTimeSpan;
@@ -58,6 +60,13 @@ namespace Smuxi.Frontend
 
         public event CommandExceptionEventHandler ExceptionEvent;
 
+        static CommandManager()
+        {
+            var asm = Assembly.GetAssembly(typeof(CommandManager));
+            var asm_name = asm.GetName(false);
+            FrontendVersion = asm_name.Version.ToString();
+        }
+
         public CommandManager(Session session)
         {
             if (session == null) {
@@ -134,8 +143,8 @@ namespace Smuxi.Frontend
             start = DateTime.UtcNow;
 
             handled = f_Session.Command(cmd);
+            IProtocolManager pm = null;
             if (!handled) {
-                IProtocolManager pm;
                 if (cmd.Chat is SessionChatModel) {
                     pm = cmd.FrontendManager.CurrentProtocolManager;
                 } else {
@@ -149,6 +158,29 @@ namespace Smuxi.Frontend
                     handled = false;
                 }
             }
+            if (!handled) {
+                var filteredCmd = IOSecurity.GetFilteredPath(cmd.Command);
+                var hooks = new HookRunner("frontend", "command-manager",
+                                           "command-" + filteredCmd);
+                hooks.EnvironmentVariables.Add("FRONTEND_VERSION", FrontendVersion);
+                hooks.Environments.Add(new ChatHookEnvironment(cmd.Chat));
+                if (pm != null) {
+                    hooks.Environments.Add(new ProtocolManagerHookEnvironment(pm));
+                }
+
+                var cmdChar = (string) f_Session.UserConfig["Interface/Entry/CommandCharacter"];
+                hooks.Commands.Add(new SessionHookCommand(f_Session, cmd.Chat, cmdChar));
+                if (pm != null) {
+                    hooks.Commands.Add(new ProtocolManagerHookCommand(pm, cmd.Chat, cmdChar));
+                }
+
+                // show time
+                hooks.Init();
+                if (hooks.HasHooks) {
+                    hooks.Run();
+                    handled = true;
+                }
+            }
             if (!handled) {
                Unknown(cmd);
             }
@@ -203,7 +235,7 @@ namespace Smuxi.Frontend
                                              cmd.CommandCharacter, output));
                 } else {
                     var msg = new MessageBuilder().AppendText(output).ToMessage();
-                    cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg);
+                    f_Session.AddMessageToFrontend(cmd, msg);
                 }
             };
 
@@ -250,7 +282,7 @@ namespace Smuxi.Frontend
                         AppendErrorText("Executing '{0}' failed with: {1}",
                                         command, ex.Message).
                         ToMessage();
-                    cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg);
+                    f_Session.AddMessageToFrontend(cmd, msg);
                 }
             }
         }
@@ -263,7 +295,7 @@ namespace Smuxi.Frontend
                 AppendEventPrefix().
                     AppendText(cmd.Parameter).
                     ToMessage();
-            cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg);
+            f_Session.AddMessageToFrontend(cmd, msg);
         }
 
         public void CommandGenerateMessages(CommandModel cmd, IChatView chat)
@@ -310,7 +342,7 @@ namespace Smuxi.Frontend
                 AppendEventPrefix().
                 AppendText(_("Unknown Command: {0}"), cmd.Command).
                 ToMessage();
-            cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg);
+            f_Session.AddMessageToFrontend(cmd, msg);
         }
 
         void NotEnoughParameters(CommandModel cmd)
@@ -319,7 +351,7 @@ namespace Smuxi.Frontend
                 AppendEventPrefix().
                 AppendText(_("Not enough parameters for {0} command"), cmd.Command).
                 ToMessage();
-            cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg);
+            f_Session.AddMessageToFrontend(cmd, msg);
         }
 
         MessageBuilder CreateMessageBuilder()
